@@ -6,6 +6,8 @@ import pymcfunc.internal as internal
 from pymcfunc.func_handler_universal import UniversalFuncHandler, UniversalRawCommands
 from pymcfunc.selectors import BedrockSelectors
 
+_b = lambda x: 'true' if x == True else 'false' if x == False else x
+
 class BedrockFuncHandler(UniversalFuncHandler):
     """The Beckrock Edition function handler.
     More info: https://pymcfunc.rtfd.io/en/latest/reference.html#pymcfunc.BedrockFuncHandler"""
@@ -15,6 +17,11 @@ class BedrockFuncHandler(UniversalFuncHandler):
         self.commands = []
         self.r = BedrockRawCommands(self)
 
+    def v(self, name: str, target: str):
+        """Creates a variable.
+        More info: https://pymcfunc.rtfd.io/en/latest/reference.html#pymcfunc.BedrockFuncHandler.v"""
+        return BedrockVariable(self, name, target)
+
 class BedrockRawCommands(UniversalRawCommands):
     """A container for raw Minecraft commands that are specially for Bedrock Edition.
     More info: https://pymcfunc.rtfd.io/en/latest/reference.html#pymcfunc.BedrockRawCommands"""
@@ -115,11 +122,11 @@ class BedrockRawCommands(UniversalRawCommands):
             rotation_facing = internal.pick_one_arg((rotation, None, 'rotation'), (facing, None, 'facing'))
             if rotation_facing is not None:
                 if facing is not None: rotation_facing = "facing "+rotation_facing
-                optionals = f"{rotation_facing} {internal.defaults((checkForBlocks, False))}"
+                optionals = f"{rotation_facing} {internal.defaults((_b(checkForBlocks), 'false'))}"
             else:
-                optionals = internal.defaults((checkForBlocks, False))
+                optionals = internal.defaults((_b(checkForBlocks), 'false'))
         else:
-            optionals = internal.defaults((checkForBlocks, False))
+            optionals = internal.defaults((_b(checkForBlocks), 'false'))
 
         cmd = f"teleport {target}{dest} {optionals}".strip()
         self.fh.commands.append(cmd)
@@ -143,7 +150,7 @@ class BedrockRawCommands(UniversalRawCommands):
         """**Syntax:** *<target> <effect> [seconds] [amplifier] [hideParticles]*\n
         More info: https://pymcfunc.rtfd.io/en/latest/reference.html#pymcfunc.BedrockRawCommands.effect_give"""
         internal.check_spaces('target', target)
-        optionals = internal.defaults((seconds, 30), (amplifier, 0), (hideParticles, False))
+        optionals = internal.defaults((seconds, 30), (amplifier, 0), (_b(hideParticles), 'false'))
 
         cmd = f"effect {target} {effect} {optionals}".strip()
         self.fh.commands.append(cmd)
@@ -305,7 +312,7 @@ class BedrockRawCommands(UniversalRawCommands):
         internal.multi_check_invalid_params(['add', 'remove', 'setdisplay'], 'mode', mode, ('objective', objective, None))
         if mode in ['add', 'remove'] and objective is None:
             raise errors.MissingError('objective', 'mode', mode)
-        internal.check_invalid_params('add', 'mode', mode, ('displayName', displayName, None), dep_mandatory=True)
+        internal.check_invalid_params('add', 'mode', mode, ('displayName', displayName, None))
         internal.check_invalid_params('setdisplay', 'mode', mode, ('slot', slot, None), dep_mandatory=True)
         if slot is not None:
             internal.options(slot, ['list', 'sidebar', 'belowname'])
@@ -407,7 +414,7 @@ class BedrockRawCommands(UniversalRawCommands):
         """**Syntax:** *ability <target> [ability] [value]*\n
         More info: https://pymcfunc.rtfd.io/en/latest/reference.html#pymcfunc.BedrockRawCommands.ability"""
         internal.reliant('ability', ability, None, 'value', value, None)
-        optionals = internal.defaults((ability, None), (value, None))
+        optionals = internal.defaults((ability, None), (_b(value), None))
         cmd = f"ability {target} {optionals}".strip()
         self.fh.commands.append(cmd)
         return cmd
@@ -477,8 +484,8 @@ class BedrockRawCommands(UniversalRawCommands):
     def alwaysday(self, lock: bool=None):
         """**Syntax:** *alwaysday [lock]*\n
         More info: https://pymcfunc.rtfd.io/en/latest/reference.html#pymcfunc.BedrockRawCommands.alwaysday"""
-        internal.defaults((lock, None))
-        cmd = f"alwaysday {lock}".strip()
+        optionals = internal.defaults((_b(lock), None))
+        cmd = f"alwaysday {optionals}".strip()
         self.fh.commands.append(cmd)
         return cmd
     daylock = alwaysday
@@ -510,7 +517,7 @@ class BedrockRawCommands(UniversalRawCommands):
         if difficulty is not None:
             internal.options(difficulty, ['peaceful', 'easy', 'normal', 'hard', 'p', 'e', 'n', 'h', 0, 1, 2, 3])
         value = internal.pick_one_arg(
-            (allow_cheats, None, 'allow_cheats'),
+            (_b(allow_cheats), None, 'allow_cheats'),
             (difficulty, None, 'difficulty'),
             optional=False
         )
@@ -650,14 +657,14 @@ class BedrockRawCommands(UniversalRawCommands):
     def globalpause(self, pause: bool):
         """**Syntax:** *globalpause <pause>*\n
         More info: https://pymcfunc.rtfd.io/en/latest/reference.html#pymcfunc.BedrockRawCommands.getspawnpoint"""
-        cmd = f"globalpause {pause}".strip()
+        cmd = f"globalpause {_b(pause)}".strip()
         self.fh.commands.append(cmd)
         return cmd
 
     def immutableworld(self, immutable: bool=None):
         """**Syntax:** *immutableworld [immutable]*\n
         More info: https://pymcfunc.rtfd.io/en/latest/reference.html#pymcfunc.BedrockRawCommands.immutableworld"""
-        optionals = internal.defaults((immutable, None))
+        optionals = internal.defaults((_b(immutable), None))
         cmd = f"immutableworld {optionals}".strip()
         self.fh.commands.append(cmd)
         return cmd
@@ -671,7 +678,7 @@ class BedrockRawCommands(UniversalRawCommands):
     def mobevent(self, event: str, value: bool=None):
         """**Syntax:** *mobevent <event> [value]*\n
         More info: https://pymcfunc.rtfd.io/en/latest/reference.html#pymcfunc.BedrockRawCommands.mobevent"""
-        optionals = internal.defaults((value, None))
+        optionals = internal.defaults((_b(value), None))
         cmd = f"mobevent {event} {optionals}".strip()
         self.fh.commands.append(cmd)
         return cmd
@@ -803,7 +810,7 @@ class BedrockRawCommands(UniversalRawCommands):
         """**Syntax:** *structure save <name> <pos1> <pos2> [includesEntities] [saveMode:disk|memory] [includesBlocks]*\n
         More info: https://pymcfunc.rtfd.io/en/latest/reference.html#pymcfunc.BedrockRawCommands.structure_save"""
         internal.options(saveMode, ['disk', 'memory'])
-        optionals = internal.defaults((includesEntities, True), (saveMode, 'disk'), (includesBlocks, True))
+        optionals = internal.defaults((_b(includesEntities), 'true'), (saveMode, 'disk'), (_b(includesBlocks), 'true'))
         cmd = f"structue save {name} {pos1} {pos2} {optionals}".strip()
         self.fh.commands.append(cmd)
         return cmd
@@ -819,9 +826,9 @@ class BedrockRawCommands(UniversalRawCommands):
         internal.options(mirror, ['x', 'z', 'xz', 'none'])
         if animationMode is not None:
             internal.options(animationMode, ['block_by_block', 'layer_by_layer'])
-            optional_list = [(rotation, '0_degrees'), (mirror, 'none'), (animationMode, None), (animationSeconds, 1), (includesEntities, True), (includesBlocks, True), (integrity, 100), (seed, None)]
+            optional_list = [(rotation, '0_degrees'), (mirror, 'none'), (animationMode, None), (animationSeconds, 1), (_b(includesEntities), 'true'), (_b(includesBlocks), 'true'), (integrity, 100), (seed, None)]
         else:
-            optional_list = [(rotation, '0_degrees'), (mirror, 'none'), (includesEntities, True), (includesBlocks, True), (integrity, 100), (seed, None)]
+            optional_list = [(rotation, '0_degrees'), (mirror, 'none'), (_b(includesEntities), 'true'), (_b(includesBlocks), 'true'), (integrity, 100), (seed, None)]
 
         optionals = internal.defaults(*optional_list)
         cmd = f"structure load {name} {pos} {optionals}".strip()
@@ -917,4 +924,116 @@ class BedrockRawCommands(UniversalRawCommands):
         More info: https://pymcfunc.rtfd.io/en/latest/reference.html#pymcfunc.BedrockRawCommands.worldbuilder"""
         self.fh.commands.append('worldbuilder')
         return 'worldbuilder'
-    wb = worldbuilder
\ No newline at end of file
+    wb = worldbuilder
+
+class BedrockVariable:
+    """Represents a variable in Bedrock Edition.
+    More info: https://pymcfunc.rtfd.io/en/latest/reference.html#pymcfunc.BedrockVariable"""
+    def __init__(self, fh, name: str, target: str):
+        self.fh = fh
+        self.name = name
+        self.target = target
+        self.fh.r.scoreboard_objectives('add', objective=name)
+        #self.fh.r.scoreboard_players('set', target=target, objective=name, count=0)
+
+    def __iadd__(self, other: Union['BedrockVariable', int]):
+        if isinstance(other, type(self)):
+            self.fh.r.scoreboard_players('operation', target=self.target, objective=self.name,
+                                         operation='+=', selector=other.target, selectorObjective=other.name)
+        if other < 0:
+            self.__isub__(other)
+        else:
+            self.fh.r.scoreboard_players('add', target=self.target, objective=self.name, count=other)
+        return self
+
+    def __isub__(self, other: Union['BedrockVariable', int]):
+        if isinstance(other, type(self)):
+            self.fh.r.scoreboard_players('operation', target=self.target, objective=self.name,
+                                         operation='-=', selector=other.target, selectorObjective=other.name)
+        elif other < 0:
+            self.__iadd__(other)
+        else:
+            self.fh.r.scoreboard_players('remove', target=self.target, objective=self.name, count=other)
+        return self
+
+    def __imul__(self, other: Union['BedrockVariable', int]):
+        if isinstance(other, type(self)):
+            self.fh.r.scoreboard_players('operation', target=self.target, objective=self.name,
+                                         operation='*=', selector=other.target, selectorObjective=other.name)
+        else:
+            temp = type(self)(self.fh, self.name+'temp', self.target)
+            temp.set(other)
+            self.fh.r.scoreboard_players('operation', target=self.target, objective=self.name,
+                                         operation='*=', selector=temp.target, selectorObjective=temp.name)
+            del temp
+        return self
+
+    def __itruediv__(self, other: Union['BedrockVariable', int]):
+        if isinstance(other, type(self)):
+            self.fh.r.scoreboard_players('operation', target=self.target, objective=self.name,
+                                         operation='*=', selector=other.target, selectorObjective=other.name)
+        else:
+            temp = type(self)(self.fh, self.name+'temp', self.target)
+            temp.set(other)
+            self.fh.r.scoreboard_players('operation', target=self.target, objective=self.name,
+                                         operation='/=', selector=temp.target, selectorObjective=temp.name)
+            del temp
+        return self
+    __ifloordiv__ = __itruediv__
+
+    def __imod__(self, other: Union['BedrockVariable', int]):
+        if isinstance(other, type(self)):
+            self.fh.r.scoreboard_players('operation', target=self.target, objective=self.name,
+                                         operation='%=', selector=other.target, selectorObjective=other.name)
+        else:
+            temp = type(self)(self.fh, self.name+'temp', self.target)
+            temp.set(other)
+            self.fh.r.scoreboard_players('operation', target=self.target, objective=self.name,
+                                         operation='%=', selector=temp.target, sselectorbjective=temp.name)
+            del temp
+        return self
+
+    def __del__(self):
+        self.fh.r.scoreboard_players('reset', target=self.target, objective=self.name)
+
+    def in_range(self, minv: int, maxv: int=None):
+        """Tests a value if it is within a certain range.
+        More info: https://pymcfunc.rtfd.io/en/latest/reference.html#pymcfunc.BedrockVariable.in_range"""
+        self.fh.r.scoreboard_players('test', target=self.target, objective=self.name, minv=minv, maxv=maxv)
+
+    def set(self, other: Union['BedrockVariable', int]):
+        """Sets this variable to a value or that of another variable
+        More info: https://pymcfunc.rtfd.io/en/latest/reference.html#pymcfunc.BedrockVariable.set"""
+        if isinstance(other, type(self)):
+            self.fh.r.scoreboard_players('operation', target=self.target, objective=self.name,
+                                         operation='=', source=other.target, sourceObjective=other.name)
+        else:
+            self.fh.r.scoreboard_players('set', target=self.target, objective=self.name, count=other)
+
+    def random(self, minv: int, maxv: int=None):
+        """Sets this variable to a random number.
+        More info: https://pymcfunc.rtfd.io/en/latest/reference.html#pymcfunc.BedrockVariable.random"""
+        self.fh.r.scoreboard_players('random', target=self.target, objective=self.name, minv=minv, maxv=maxv)
+
+    def higher(self, other: 'BedrockVariable'):
+        """Sets this variable to the higher of the two variables.
+        More info: https://pymcfunc.rtfd.io/en/latest/reference.html#pymcfunc.BedrockVariable.higher"""
+        self.fh.r.scoreboard_players('operation', target=self.target, objective=self.name,
+                                     operation='>', selector=other.target, selectorObjective=other.name)
+
+    def lower(self, other: 'BedrockVariable'):
+        """Sets this variable to the lower of the two variables.
+        More info: https://pymcfunc.rtfd.io/en/latest/reference.html#pymcfunc.BedrockVariable.lower"""
+        self.fh.r.scoreboard_players('operation', target=self.target, objective=self.name,
+                                     operation='<', selector=other.target, selectorObjective=other.name)
+
+    def swap(self, other: 'BedrockVariable'):
+        """Swaps the value of the two variables.
+        More info: https://pymcfunc.rtfd.io/en/latest/reference.html#pymcfunc.BedrockVariable.swap"""
+        self.fh.r.scoreboard_players('operation', target=self.target, objective=self.name,
+                                     operation='><', selector=other.target, selectorObjective=other.name)
+
+    def show(self, slot: str, sortOrder: str=None):
+        """Shows the variable in a slot.
+        More info: https://pymcfunc.rtfd.io/en/latest/reference.html#pymcfunc.BedrockVariable.show"""
+        self.fh.r.scoreboard_objectives('setdisplay', slot=slot, objective=self.name, sortOrder=sortOrder)
\ No newline at end of file
