@@ -1,11 +1,14 @@
 import json
 import re
+from typing import Union
 
 import pymcfunc.errors as errors
 import pymcfunc.internal as internal
 from pymcfunc.func_handler_universal import UniversalFuncHandler, UniversalRawCommands
 from pymcfunc.selectors import JavaSelectors
 
+_b = lambda x: 'true' if x == True else 'false' if x == False else x
+
 class JavaFuncHandler(UniversalFuncHandler):
     """The Java Edition function handler.
     More info: https://pymcfunc.rtfd.io/en/latest/reference.html#pymcfunc.JavaFuncHandler"""
@@ -15,6 +18,11 @@ class JavaFuncHandler(UniversalFuncHandler):
         self.commands = []
         self.r = JavaRawCommands(self)
 
+    def v(self, name: str, target: str, trigger: bool=False):
+        """Creates a variable.
+        More info: https://pymcfunc.rtfd.io/en/latest/reference.html#pymcfunc.JavaFuncHandler.v"""
+        return JavaVariable(self, name, target, trigger=trigger)
+
 class JavaRawCommands(UniversalRawCommands):
     def setblock(self, pos: str, block: str, mode="replace"):
         """**Syntax:** *setblock <pos> <block> [mode:destroy|keep|replace]*\n
@@ -137,7 +145,7 @@ class JavaRawCommands(UniversalRawCommands):
         """**Syntax:** *effect give <target> <effect> [seconds] [amplifier] [hideParticles]*\n
         More info: https://pymcfunc.rtfd.io/en/latest/reference.html#pymcfunc.JavaRawCommands.effect_give"""
         internal.check_spaces('target', target)
-        optionals = internal.defaults((seconds, 30), (amplifier, 0), (hideParticles, False))
+        optionals = internal.defaults((seconds, 30), (amplifier, 0), (_b(hideParticles), 'false'))
 
         cmd = f"effect give {target} {effect} {optionals}".strip()
         self.fh.commands.append(cmd)
@@ -182,12 +190,12 @@ class JavaRawCommands(UniversalRawCommands):
         self.fh.commands.append(cmd)
         return cmd
 
-    def schedule(self, name: str, clear: bool=False, duration: str=None, mode: str="replace"):
+    def schedule(self, name: str, clear: bool=False, duration: int=None, mode: str="replace"):
         """**Syntax:** *schedule ...*
         * *function <name> <duration> [mode:append|replace]*
         * *clear <name>*\n
         https://pymcfunc.rtfd.io/en/latest/reference.html#pymcfunc.JavaRawCommands.schedule"""
-        internal.check_invalid_params(False, 'clear', clear, ('duration', duration, None), dep_mandatory=True)
+        internal.check_invalid_params('false', 'clear', _b(clear), ('duration', duration, None), dep_mandatory=True)
         if clear:
             cmd = f"schedule clear {name}".strip()
         else:
@@ -253,7 +261,7 @@ class JavaRawCommands(UniversalRawCommands):
             maxHeight = "under "+maxHeight+" "
         else:
             maxHeight = ""
-        cmd = f"spreadplayers {center} {dist} {maxRange} {maxHeight}{respectTeams} {target}"
+        cmd = f"spreadplayers {center} {dist} {maxRange} {maxHeight}{_b(respectTeams)} {target}"
         self.fh.commands.append(cmd)
         return cmd
 
@@ -290,7 +298,7 @@ class JavaRawCommands(UniversalRawCommands):
             raise errors.MissingError('objective', 'mode', mode)
         internal.check_invalid_params('add', 'mode', mode, ('criterion', criterion, None), dep_mandatory=True)
         internal.multi_check_invalid_params(['add', 'modify_displayname'], 'mode', mode, ('displayName', displayName, None))
-        if mode != 'add' and displayName is None:
+        if mode == 'modify_displayname' and displayName is None:
             raise errors.MissingError('displayName', 'mode', mode)
         internal.check_invalid_params('modify_rendertype', 'mode', mode, ('renderType', renderType, None), dep_mandatory=True)
         if renderType is not None:
@@ -401,7 +409,7 @@ class JavaRawCommands(UniversalRawCommands):
             "store": str (result|success),
             "mode": str (block|bossbar|entity|score|storage),
             "pos": str (when mode=block),
-            "entity": str (when mode=entity,score,storage),
+            "target": str (when mode=entity,score,storage),
             "id": str (when mode=bossbar),
             "value": str (value|max when mode=bossbar),
             "objective": str (when mode=score),
@@ -511,10 +519,10 @@ class JavaRawCommands(UniversalRawCommands):
                 elif v['mode'] == "score":
                     internal.options(v['comparer'], ['<', '<=', '=', '>', '>=', 'matches'])
                     internal.check_spaces('target', v['target'])
-                    if v['comparer'] == "range":
-                        return f"{prefix} matches {v['range']} "
+                    if v['comparer'] == "matches":
+                        return f"{prefix} {v['target']} {v['objective']} matches {v['range']} "
                     else:
-                        return f"{prefix} {v['comparer']} {v['source']} {v['sourceObjective']} "
+                        return f"{prefix} {v['target']} {v['objective']} {v['comparer']} {v['source']} {v['sourceObjective']} "
             s_unless = s_if_
 
             @staticmethod
@@ -754,7 +762,7 @@ class JavaRawCommands(UniversalRawCommands):
             (target, None, 'target'),
             (style, None, 'style'),
             (value, None, 'value'),
-            (visible, None, 'visible')
+            (_b(visible), None, 'visible')
         )
         cmd = f"bossbar set {barId} {mode} {v}".strip()
         self.fh.commands.append(cmd)
@@ -1213,4 +1221,166 @@ class JavaRawCommands(UniversalRawCommands):
         cmd = f"worldborder warning {middle} {value}".strip()
         cmd = "worldborder get"
         self.fh.commands.append(cmd)
-        return cmd
\ No newline at end of file
+        return cmd
+
+class JavaVariable:
+    """Represents a variable in Java Edition.
+    More info: https://pymcfunc.rtfd.io/en/latest/reference.html#pymcfunc.JavaVariable"""
+    def __init__(self, fh, name: str, target: str, trigger: bool=False):
+        self.fh = fh
+        self.name = name
+        self.target = target
+        criterion = 'dummy' if not trigger else 'trigger'
+        self.fh.r.scoreboard_objectives('add', objective=name, criterion=criterion)
+        #self.fh.r.scoreboard_players('set', target=target, objective=name, score=0)
+
+    def __iadd__(self, other: Union['JavaVariable', int]):
+        if isinstance(other, type(self)):
+            self.fh.r.scoreboard_players('operation', target=self.target, objective=self.name,
+                                         operation='+=', source=other.target, sourceObjective=other.name)
+        if other < 0:
+            self.__isub__(other)
+        else:
+            self.fh.r.scoreboard_players('add', target=self.target, objective=self.name, score=other)
+        return self
+
+    def __isub__(self, other: Union['JavaVariable', int]):
+        if isinstance(other, type(self)):
+            self.fh.r.scoreboard_players('operation', target=self.target, objective=self.name,
+                                         operation='-=', source=other.target, sourceObjective=other.name)
+        elif other < 0:
+            self.__iadd__(other)
+        else:
+            self.fh.r.scoreboard_players('remove', target=self.target, objective=self.name, score=other)
+        return self
+
+    def __imul__(self, other: Union['JavaVariable', int]):
+        if isinstance(other, type(self)):
+            self.fh.r.scoreboard_players('operation', target=self.target, objective=self.name,
+                                         operation='*=', source=other.target, sourceObjective=other.name)
+        else:
+            temp = type(self)(self.fh, self.name+'temp', self.target)
+            temp.set(other)
+            self.fh.r.scoreboard_players('operation', target=self.target, objective=self.name,
+                                         operation='*=', source=temp.target, sourceObjective=temp.name)
+            del temp
+        return self
+
+    def __itruediv__(self, other: Union['JavaVariable', int]):
+        if isinstance(other, type(self)):
+            self.fh.r.scoreboard_players('operation', target=self.target, objective=self.name,
+                                         operation='*=', source=other.target, sourceObjective=other.name)
+        else:
+            temp = type(self)(self.fh, self.name+'temp', self.target)
+            temp.set(other)
+            self.fh.r.scoreboard_players('operation', target=self.target, objective=self.name,
+                                         operation='/=', source=temp.target, sourceObjective=temp.name)
+            del temp
+        return self
+    __ifloordiv__ = __itruediv__
+
+    def __imod__(self, other: Union['JavaVariable', int]):
+        if isinstance(other, type(self)):
+            self.fh.r.scoreboard_players('operation', target=self.target, objective=self.name,
+                                         operation='%=', source=other.target, sourceObjective=other.name)
+        else:
+            temp = type(self)(self.fh, self.name+'temp', self.target)
+            temp.set(other)
+            self.fh.r.scoreboard_players('operation', target=self.target, objective=self.name,
+                                         operation='%=', source=temp.target, sourceObjective=temp.name)
+            del temp
+        return self
+
+    def __del__(self):
+        self.fh.r.scoreboard_players('reset', target=self.target, objective=self.name)
+
+    @staticmethod
+    def _comparers(self, other, mode):
+        if isinstance(other, type(self)):
+            return {
+                'mode': 'score',
+                'target': self.target,
+                'objective': self.name,
+                'comparer': mode,
+                'source': other.target,
+                'sourceObjective': other.name
+            }
+        else:
+            rangeTemplates = {
+                '=': f'{other}',
+                '<': f'..{other}',
+                '<=': f'..{other-1}',
+                '>': f'{other}..',
+                '>=': f'{other+1}',
+            } if not isinstance(other, str) else {
+                '=': f'{other}'
+            }
+            return {
+                'mode': 'score',
+                'target': self.target,
+                'objective': self.name,
+                'comparer': 'matches',
+                'range': rangeTemplates[mode]
+            }
+
+    def __eq__(self, other: Union['JavaVariable', int]):
+        return self._comparers(self, other, '=')
+
+    def __lt__(self, other: Union['JavaVariable', int]):
+        return self._comparers(self, other, '<')
+
+    def __le__(self, other: Union['JavaVariable', int]):
+        return self._comparers(self, other, '<=')
+
+    def __gt__(self, other: Union['JavaVariable', int]):
+        return self._comparers(self, other, '>')
+
+    def __ge__(self, other: Union['JavaVariable', int]):
+        return self._comparers(self, other, '>=')
+
+    def in_range(self, r: Union[str, int]):
+        """For use in JavaRawCommands.execute(). Finds whether this variable is in a specified range.
+        More info: https://pymcfunc.rtfd.io/en/latest/reference.html#pymcfunc.JavaVariable.in_range"""
+        return self.__eq__(r)
+
+    def store(self, mode: str):
+        """For use in JavaRawCommands.execute(). Stores a result or success in this variable.
+        More info: https://pymcfunc.rtfd.io/en/latest/reference.html#pymcfunc.JavaVariable.store"""
+        return {
+            'store': mode,
+            'mode': 'score',
+            'target': self.target,
+            'objective': self.name
+        }
+
+    def set(self, other: Union['JavaVariable', int]):
+        """Sets this variable to a value or that of another variable.
+        More info: https://pymcfunc.rtfd.io/en/latest/reference.html#pymcfunc.JavaVariable.set"""
+        if isinstance(other, type(self)):
+            self.fh.r.scoreboard_players('operation', target=self.target, objective=self.name,
+                                         operation='=', source=other.target, sourceObjective=other.name)
+        else:
+            self.fh.r.scoreboard_players('set', target=self.target, objective=self.name, score=other)
+
+    def higher(self, other: 'JavaVariable'):
+        """Sets this variable to the higher of the two variables.
+        More info: https://pymcfunc.rtfd.io/en/latest/reference.html#pymcfunc.JavaVariable.higher"""
+        self.fh.r.scoreboard_players('operation', target=self.target, objective=self.name,
+                                     operation='>', source=other.target, sourceObjective=other.name)
+
+    def lower(self, other: 'JavaVariable'):
+        """Sets this variable to the lower of the two variables.
+        More info: https://pymcfunc.rtfd.io/en/latest/reference.html#pymcfunc.JavaVariable.lower"""
+        self.fh.r.scoreboard_players('operation', target=self.target, objective=self.name,
+                                     operation='<', source=other.target, sourceObjective=other.name)
+
+    def swap(self, other: 'JavaVariable'):
+        """Swaps the value of the two variables.
+        More info: https://pymcfunc.rtfd.io/en/latest/reference.html#pymcfunc.JavaVariable.swap"""
+        self.fh.r.scoreboard_players('operation', target=self.target, objective=self.name,
+                                     operation='><', source=other.target, sourceObjective=other.name)
+
+    def show(self, slot: str):
+        """Shows the variable in a slot.
+        More info: https://pymcfunc.rtfd.io/en/latest/reference.html#pymcfunc.JavaVariable.show"""
+        self.fh.r.scoreboard_objectives('setdisplay', objective=self.name, slot=slot)
\ No newline at end of file
