@@ -6,6 +6,8 @@ import pymcfunc.errors as errors
 import pymcfunc.internal as internal
 from pymcfunc.selectors import UniversalSelectors
 
+_b = lambda x: 'true' if x == True else 'false' if x == False else x
+
 class UniversalFuncHandler:
     """The function handler which includes commands that are the same for both Java and Bedrock edition.
     More info: https://pymcfunc.rtfd.io/en/latest/reference.html#pymcfunc.UniversalFuncHandler"""
@@ -20,6 +22,16 @@ class UniversalFuncHandler:
         for i in self.commands:
             yield i
 
+    def clear(self):
+        """Clears the command list.
+        More info: https://pymcfunc.rtfd.io/en/latest/reference.html#pymcfunc.UniversalFuncHandler.clear"""
+        self.commands = []
+
+    def comment(self, comment: str):
+        """Adds a comment.
+        More info: https://pymcfunc.rtfd.io/en/latest/reference.html#pymcfunc.UniversalFuncHandler.comment"""
+        self.commands.append('# '+comment.strip())
+
 class UniversalRawCommands:
     """A container for raw Minecraft commands that are the same for both Java and Bedrock.
     More info: https://pymcfunc.rtfd.io/en/latest/reference.html#pymcfunc.UniversalRawCommands"""
@@ -43,7 +55,7 @@ class UniversalRawCommands:
     w = tell
     msg = tell
 
-    def tellraw(self, target: str, message: dict):
+    def tellraw(self, target: str, message: Union[dict, list]):
         """**Syntax:** *tellraw <target> <message>*\n
         More info: https://pymcfunc.rtfd.io/en/latest/reference.html#pymcfunc.UniversalRawCommands.tellraw"""
         internal.check_spaces('target', target)
@@ -51,7 +63,7 @@ class UniversalRawCommands:
         self.fh.commands.append(cmd)
         return cmd
 
-    def title(self, target: str, mode: str, text: Union[str, dict]=None, fadeIn: int=None, stay: int=None, fadeOut: int=None):
+    def title(self, target: str, mode: str, text: Union[str, Union[dict, list]]=None, fadeIn: int=None, stay: int=None, fadeOut: int=None):
         """**Syntax:** *title <target> ...*\n
         * *... <mode\:clear|reset>*
         * *... <mode\:title|subtitle|actionbar> <text>*
